package ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import ag.Logic;

public class MainFrame {

	private static final int WIDTH = 800;

	private static final int HEIGHT = 600;

	private static final String TITLE = "Girafas - Por Flavio Roberto, Vinicius Ferraz e Flavio Vinicius";

	private JFrame frame;

	private JMenuBar menuBar;
	
	private GirafasPanel girafas;

	private OpcoesPanel opcoes;

	private Logic proximaLogica;

	public MainFrame() {
		frame = new JFrame(TITLE);
		menuBar = new JMenuBar();
		opcoes = new OpcoesPanel(this);
		proximaLogica = opcoes.getLogic();
		
		criarFrame();
		criarMenu();
	}

	private void novoGirafas() {
		// creates the panel
		girafas = new GirafasPanel(WIDTH, HEIGHT, proximaLogica);
		// add panel
		frame.getContentPane().add(girafas);
	}

	private void criarFrame() {
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setJMenuBar(menuBar);
		frame.pack();
		frame.setSize(WIDTH + frame.getInsets().left + frame.getInsets().right,
				HEIGHT + 20 + frame.getInsets().top + frame.getInsets().bottom);
		frame.setResizable(false);
	}
	
	public void proximaLogica(Logic logic) {
		this.proximaLogica = logic;
	}


	private void criarMenu() {
		JMenu menuArquivo = new JMenu("Arquivo");
		menuArquivo.setMnemonic('A');
		criarNovoItem(menuArquivo, "Iniciar", 'I', new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					iniciar();
				}
			}
		);
		criarNovoItem(menuArquivo, "Parar", 'P', new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					parar();
				}
			}
		);
		criarNovoItem(menuArquivo, "Sair", 'S', new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					System.exit(0);
				}
			}
		);
		
		JMenu menuConfigurar = new JMenu("Configurar");
		menuConfigurar.setMnemonic('C');
		criarNovoItem(menuConfigurar, "Configurar", 'C', new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					opcoes.aparecer();
				}
			}
		);
		
		menuBar.add(menuArquivo);
		menuBar.add(menuConfigurar);
	}

	private void criarNovoItem(JMenu menuPai, String nome, char atalho, ActionListener l) {
		JMenuItem item = new JMenuItem(nome);
		item.setMnemonic(atalho);
		item.addActionListener(l);
		menuPai.add(item);
	}
	
	public void iniciar() {
		if (girafas == null) {
			novoGirafas();
			girafas.iniciar();
		}
	}

	protected void parar() {
		if (girafas != null) {
			girafas.parar();
			girafas = null;
		}
	}
	
	public void aparecer() {
		frame.setVisible(true);		
	}
}