package objects;

import java.awt.Graphics;
import java.io.File;

import javax.swing.ImageIcon;

import base.AbstractObject;

public final class GirafaObject extends AbstractObject implements Comparable<GirafaObject>{

	private static final int TAMANHO_CROMOSSOMO = 6;

	private WorldLogic world;

	private ImageIcon cabecaDireita;
	private ImageIcon pescocoDireita;
	private ImageIcon corpoDireita;

	private ImageIcon cabecaEsquerda;
	private ImageIcon pescocoEsquerda;
	private ImageIcon corpoEsquerda;

	private int alturaDaGirafa;
	private int tamanhoDoPescoco;
	private int direcao;
	private int satisfacao;
	private int perdeSatisfacao;
	private int satisfacaoAoComer;

	public GirafaObject(WorldLogic world, int satisfacaoInicial, int perdeSatisfacao, int satisfacaoAoComer, int tamanhoDoPescoco) {
		this.world = world;

		this.cabecaDireita = new ImageIcon("imgs" + File.separator + "girafa" + File.separator + "cabeca3.gif");
		this.pescocoDireita = new ImageIcon("imgs" + File.separator + "girafa" + File.separator + "pescoco3.gif");
		this.corpoDireita = new ImageIcon("imgs" + File.separator + "girafa" + File.separator + "corpo3.gif");

		this.cabecaEsquerda = new ImageIcon("imgs" + File.separator + "girafa" + File.separator + "cabeca3Esquerda.gif");
		this.pescocoEsquerda = new ImageIcon("imgs" + File.separator + "girafa" + File.separator + "pescoco3Esquerda.gif");
		this.corpoEsquerda = new ImageIcon("imgs" + File.separator + "girafa" + File.separator + "corpo3Esquerda.gif");

		this.tamanhoDoPescoco = tamanhoDoPescoco & ~(~0 << TAMANHO_CROMOSSOMO);
		this.alturaDaGirafa = this.cabecaDireita.getIconHeight() + this.tamanhoDoPescoco * this.pescocoDireita.getIconHeight() + this.corpoDireita.getIconHeight();
		this.direcao = geraDirecaoInicial();
		this.satisfacao = satisfacaoInicial;
		this.perdeSatisfacao = perdeSatisfacao;
		this.satisfacaoAoComer = satisfacaoAoComer;
		
		super.width = this.corpoDireita.getIconWidth();
		super.height = this.alturaDaGirafa;
		super.x = geraPosicao();
		super.y = this.world.getHeight() - this.alturaDaGirafa;
	}

	private int geraPosicao() {
		return (int) (Math.random() * (this.world.getWidth() - this.getWidth()));
	}

	private int geraDirecaoInicial() {
		return (int) (Math.random() * 2) * 2 - 1;
	}

	private void atualizaDirecao() {
		int random = (int) (Math.random() * 100);
		if (random < 1) direcao *= -1;
	}

	@Override
	public void update() {
		if (this.getX() + direcao * 5 < 0 || this.getX() + this.getWidth() + direcao * 5 >= this.world.getWidth()) {
			direcao *= -1;
		}

		this.x += direcao * 5;
		atualizaDirecao();

		this.satisfacao = Math.max(0, this.satisfacao - this.perdeSatisfacao);
		if (this.world.podeComer(this)) {
			this.satisfacao += this.satisfacaoAoComer;
		}
	}

	@Override
	public void paint(Graphics g) {
		if(this.direcao == 1) {
			this.cabecaDireita.paintIcon(null, g, this.getX() + 27, this.getY());
			for (int i = 0; i < this.tamanhoDoPescoco; i++) {
				this.pescocoDireita.paintIcon(null, g, this.getX() + 28,
						(this.getY() + this.cabecaDireita.getIconHeight()) + i * this.pescocoDireita.getIconHeight());
			}
			this.corpoDireita.paintIcon(null, g, this.getX(),
					(this.getY() + this.cabecaDireita.getIconHeight()) + this.tamanhoDoPescoco * this.pescocoDireita.getIconHeight());

			g.drawString(this.satisfacao+"", this.getX()+50, this.getY());
		}else {
			this.cabecaEsquerda.paintIcon(null, g, this.getX() -5, this.getY());
			for (int i = 0; i < this.tamanhoDoPescoco; i++) {
				this.pescocoEsquerda.paintIcon(null, g, this.getX() + 1,
						(this.getY() + this.cabecaDireita.getIconHeight()) + i * this.pescocoDireita.getIconHeight());
			}
			this.corpoEsquerda.paintIcon(null, g, this.getX(),
					(this.getY() + this.cabecaDireita.getIconHeight()) + this.tamanhoDoPescoco * this.pescocoDireita.getIconHeight());

			g.drawString(this.satisfacao+"", this.getX()+50, this.getY());
		}
	}

	public String getCromossomo() {
		String s = Integer.toBinaryString(this.tamanhoDoPescoco);
		return str("0", TAMANHO_CROMOSSOMO - s.length()) + s;
	}

	private String str(String s, int n) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < n; i++) {
			sb.append(s);
		}
		return sb.toString();
	}

	public int compareTo(GirafaObject outro) {
		return outro.satisfacao- this.satisfacao;
	}

}
