package objects;
import java.awt.Graphics;
import java.util.LinkedList;
import java.util.List;

import ag.EvoluidorNovo;
import ag.Logic;
import base.AbstractObject;
import base.WorldObject;

public class WorldLogic extends AbstractObject {
	
	private int numIntervalosPassados;
	
	private WorldObject fundo;
	
	private boolean isOver;
	
	private List<GirafaObject> girafas;
	
	private List<ArvoreObject> arvores;
	
	private EvoluidorNovo util;
	
	private int geracaoAtual;

	private final Logic logic;

	private Object lock = new Object();
	
	public WorldLogic(int width, int height, Logic logic) {
		super(0, 0, width, height);
		
		this.logic = logic;
		this.fundo = new FundoObject(this);
		
		girafas = geraGirafas();
		arvores = geraArvores();
		
		numIntervalosPassados = 0;
		
		this.isOver = false;
		
		util = new EvoluidorNovo(this, logic);
		geracaoAtual = 1;
	}
	
	public boolean isOver() {
		return this.isOver;
	}
	
	public void forcarParada() {
		this.isOver = true;
	}
	
	private List<GirafaObject> geraGirafas() {
		List<GirafaObject> _girafas = new LinkedList<GirafaObject>();
		List<Integer> valores = this.geraNumeros(logic.getMediaTamGirafas(), logic.getVarTamGirafas(), logic.getNumGirafas());
		
		for (int i : valores) {
			_girafas.add(new GirafaObject(this, logic.getSatisfacaoInicial(), logic.getPerdaSatisfacaoTempo(), logic.getSatisfacaoAoComer(), i));
		}
		
		return _girafas;
	}
	
	private List<ArvoreObject> geraArvores() {
		List<ArvoreObject> _arvore = new LinkedList<ArvoreObject>();
		List<Integer> valores = this.geraNumeros(logic.getMediaTamArvores(), logic.getVarTamArvores(), logic.getNumArvores());
		
		for (int i : valores) {
			_arvore.add(new ArvoreObject(this, i));
		}
		
		return _arvore;
	}
	
	private List<Integer> geraNumeros(int media, int variancia, int quantidade) {
		List<Integer> lista = new LinkedList<Integer>();
		
		for (int i = 0; i < quantidade; i++) {
			int delta = (int) (Math.random() * (2 * variancia + 1)) - variancia;
			lista.add(media + delta);
		}
		
		return lista;
	}
	
	public boolean podeComer(GirafaObject girafa) {
		int meioDaGirafa = girafa.getX() + girafa.getWidth() / 2;
		for (ArvoreObject o : this.arvores) {
			if (meioDaGirafa > o.getX() + o.getWidth() / 2 - 20 && meioDaGirafa < o.getX() + o.getWidth() / 2 + 20
					&& girafa.getHeight() >= o.getHeight() - 200 && girafa.getHeight() < o.getHeight()) {
				return true;
			}
		}
		return false;
	}

	@Override
	public void update() {
		synchronized (lock ) {
			numIntervalosPassados = (numIntervalosPassados + 1 ) % logic.getNumIntervalos();
			
			if(this.numIntervalosPassados == 0) {
				List<GirafaObject> novaPopulacao = util.evolua(this.girafas, logic.getPercentagem(), logic.getMutProb());
				this.girafas = novaPopulacao;
				this.geracaoAtual++;
			}
			
			this.fundo.update();
			this.updateObjetos(arvores);
			this.updateObjetos(girafas);
			
			if (this.geracaoAtual == logic.getNumGeracoes()) this.isOver = true;
		}
	}

	@Override
	public void paint(Graphics g) {
		synchronized (lock) {
			this.fundo.paint(g);
			this.paintObjetos(arvores, g);
			this.paintObjetos(girafas, g);
			g.drawString("Geracao: "+geracaoAtual, 0, 13);
		}
	}

	private void updateObjetos(List<? extends WorldObject> objetos) {
		for (WorldObject o : objetos) {
			o.update();
		}
	}

	private void paintObjetos(List<? extends WorldObject> objetos, Graphics g) {
		for (WorldObject o : objetos) {
			o.paint(g);
		}
	}
}