package ui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ag.Heuristica1;
import ag.HeuristicaStrategy;
import ag.Logic;
import ag.Reprodutor1;
import ag.ReprodutorStrategy;
import ag.Selecionador1;
import ag.SelecionadorStrategy;

public class OpcoesPanel {
	
	private static final int STASIZE = 500;
	private static final String GIRAFAS_DESC = "Configurar opcoes relacionadas as Girafas";
	private static final String ARVORES_DESC = "Configurar opcoes relacionadas as arvores";
	private static final String HEURISTICA_DESC = "Configurar opcoes relacionadas a Heuristica";
	private static final String SELETOR_DESC = "Configurar opcoes relacionadas ao Seletor";
	private static final String REPRODUTOR_DESC = "Configurar opcoes relacionadas ao Reprodutor";
	private static final String OUTROS_DESC = "Configurar outras opcoes";
	
    private JFrame frameOpcoes;
    
	private JPanel configPanel;
	
	private Logic currentLogic;

	private Logic oldLogic;
	
	private MainFrame main;
	
    public OpcoesPanel(MainFrame mainFrame) {
    	main = mainFrame;
    	
		currentLogic = new Logic();
    	oldLogic = new Logic();

	    frameOpcoes = new JFrame("Opcoes");
        configPanel = new JPanel(new BorderLayout());

        JPanel girafasPanel = criarGirafasPanel();
        JPanel arvoresPanel = criarArvoresPanel();
        JPanel heuristicaPanel = criarRadioPanel(HEURISTICA_DESC, "Heuristica", 3, new HeuristicaFactory());
        JPanel seletorPanel = criarRadioPanel(SELETOR_DESC, "Seletor", 3, new SeletorFactory());
        JPanel reprodutorPanel = criarRadioPanel(REPRODUTOR_DESC, "Reprodutor", 3, new ReprodutorFactory());
        JPanel outrosPanel = criarOutrosPanel();
        
        JTabbedPane tabbedPane = new JTabbedPane();
        tabbedPane.addTab("Girafas", null,
        		girafasPanel,
                GIRAFAS_DESC);
        tabbedPane.addTab("Arvores", null,
        		arvoresPanel,
        		ARVORES_DESC);
        tabbedPane.addTab("Heuristica", null,
        		heuristicaPanel,
        		HEURISTICA_DESC);
        tabbedPane.addTab("Seletor", null,
        		seletorPanel,
        		SELETOR_DESC);
        tabbedPane.addTab("Reprodutor", null,
        		reprodutorPanel,
        		REPRODUTOR_DESC);
        tabbedPane.addTab("Outras", null,
        		outrosPanel,
        		OUTROS_DESC);
        
        configPanel.add(tabbedPane, BorderLayout.CENTER);
        
        JPanel buttonPanel = new JPanel(new FlowLayout());
        
        JButton salvar = new JButton("Salvar");
        JButton cancelar = new JButton("Cancelar");
        
        cancelar.addActionListener(new ActionListener() {
        		public void actionPerformed(ActionEvent e) {
        			currentLogic = new Logic(oldLogic);
        			frameOpcoes.setVisible(false);
        		}
			}
        );
        
        salvar.addActionListener(new ActionListener() {
    			public void actionPerformed(ActionEvent e) {
    				main.proximaLogica(currentLogic);
    				frameOpcoes.setVisible(false);
    			}
			}
        );
        
        buttonPanel.add(salvar);
        buttonPanel.add(cancelar);
        
        configPanel.add(buttonPanel, BorderLayout.PAGE_END);
        configPanel.setOpaque(true);
        
        frameOpcoes.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
	    frameOpcoes.setContentPane(configPanel);
	    frameOpcoes.pack();
	    frameOpcoes.setResizable(false);
    }

	private JPanel criarGirafasPanel() {
    	JPanel arvoresPanel = new JPanel(new BorderLayout());
    	
    	arvoresPanel.add(new JLabel(GIRAFAS_DESC), BorderLayout.NORTH);
    	
    	JPanel opcoesPanel = new JPanel(new GridLayout(6, 2, 3, 3));
    	
    	opcoesPanel.add(new JLabel("Numero de Girafas: "));
    	JSpinner numGirafas = new JSpinner(new SpinnerNumberModel(currentLogic.getNumGirafas(), 2, 1000, 1));
    	numGirafas.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					int novoValor = (Integer)spinner.getValue();
					
					if (novoValor >= 2) {
						currentLogic.setNumGirafas(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser maior ou igual a dois",
								"Erro!!", JOptionPane.ERROR_MESSAGE);
					}
				}
    		}
    	);
    	opcoesPanel.add(numGirafas);
    	
    	opcoesPanel.add(new JLabel("Media do Tamanho: "));
    	JSpinner mediaTamGirafas = new JSpinner(new SpinnerNumberModel(currentLogic.getMediaTamGirafas(), 0, 1000, 1));
    	mediaTamGirafas.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					int novoValor = (Integer)spinner.getValue();
					
					if (novoValor > 0) {
						currentLogic.setMediaTamGirafas(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
							"Valor deve ser positivo maior que zero", "Erro!!",
							JOptionPane.ERROR_MESSAGE);
					}
				}
    		}
    	);
    	opcoesPanel.add(mediaTamGirafas);
    	
    	opcoesPanel.add(new JLabel("Variancia do Tamanho: "));
    	JSpinner varTam = new JSpinner(new SpinnerNumberModel(currentLogic.getVarTamGirafas(), 0, 1000, 1));
    	varTam.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setVarTamGirafas(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
							"Valor deve ser positivo maior que zero", "Erro!!",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(varTam);
    	
    	opcoesPanel.add(new JLabel("Satisfacao inicial: "));
    	JSpinner sats = new JSpinner(new SpinnerNumberModel(currentLogic.getSatisfacaoInicial(), 0, 1000, 1));
    	sats.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setSatisfacaoInicial(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
							"Valor deve ser positivo maior que zero", "Erro!!",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(sats);
    	
    	opcoesPanel.add(new JLabel("Ganho de Satisfacao: "));
    	JSpinner insats = new JSpinner(new SpinnerNumberModel(currentLogic.getSatisfacaoAoComer(), 0, 1000, 1));
    	insats.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setSatisfacaoAoComer(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
							"Valor deve ser positivo", "Erro!!",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(insats);
    	
    	opcoesPanel.add(new JLabel("Perda de Satisfacao: "));
    	JSpinner desats = new JSpinner(new SpinnerNumberModel(currentLogic.getPerdaSatisfacaoTempo(), 0, 1000, 1));
    	desats.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setPerdaSatisfacaoTempo(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
							"Valor deve ser positivo", "Erro!!",
							JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(desats);
    	
    	arvoresPanel.add(opcoesPanel, BorderLayout.CENTER);
    	
        //Sim, isso é uma STA.
        arvoresPanel.add(Box.createHorizontalStrut(STASIZE), BorderLayout.SOUTH);
        
    	return arvoresPanel;
    }

    private JPanel criarArvoresPanel() {
    	JPanel arvoresPanel = new JPanel(new BorderLayout());
    	
    	arvoresPanel.add(new JLabel(ARVORES_DESC), BorderLayout.NORTH);
    	
    	JPanel opcoesPanel = new JPanel(new GridLayout(6, 2, 2, 2));
    	
    	opcoesPanel.add(new JLabel("Numero de Arvores: "));
    	JSpinner numArvores = new JSpinner(new SpinnerNumberModel(currentLogic.getNumArvores(), 0, 1000, 1));
    	numArvores.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setNumArvores(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser positivo", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(numArvores);

    	
    	opcoesPanel.add(new JLabel("Media do Tamanho: "));
    	JSpinner medTamArvores = new JSpinner(new SpinnerNumberModel(currentLogic.getMediaTamArvores(), 0, 1000, 1));
    	medTamArvores.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setMediaTamArvores(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser positivo", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(medTamArvores);
    	
    	opcoesPanel.add(new JLabel("Variancia do Tamanho: "));
    	JSpinner varTamArvores = new JSpinner(new SpinnerNumberModel(currentLogic.getVarTamArvores(), 0, 1000, 1));
    	varTamArvores.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setVarTamArvores(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser positivo", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(varTamArvores);
    	
    	arvoresPanel.add(opcoesPanel, BorderLayout.CENTER);
    	
        //Sim, isso eh uma STA.
        arvoresPanel.add(Box.createHorizontalStrut(STASIZE), BorderLayout.SOUTH);
        
    	return arvoresPanel;
    }
    
    private JPanel criarOutrosPanel() {
    	JPanel arvoresPanel = new JPanel(new BorderLayout());
    	
    	arvoresPanel.add(new JLabel(OUTROS_DESC), BorderLayout.NORTH);
    	
    	JPanel opcoesPanel = new JPanel(new GridLayout(6, 2, 2, 2));
    	
    	opcoesPanel.add(new JLabel("Probabilidade de Mutacao: "));
    	JSpinner mutProb = new JSpinner(new SpinnerNumberModel(currentLogic.getMutProb(), 0.0, 1.0, 0.01));
    	mutProb.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					double novoValor = (Double)spinner.getValue();
					if (novoValor > 0.0 && novoValor < 1.0) {
						currentLogic.setMutProb(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser maior que zero e menor do que um", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(mutProb);
    	
    	opcoesPanel.add(new JLabel("Percentagem de Mutacoes: "));
    	JSpinner percMut = new JSpinner(new SpinnerNumberModel(currentLogic.getPercentagem(), 0.0, 1.0, 0.01));
    	percMut.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					double novoValor = (Double)spinner.getValue();
					if (novoValor > 0.0 && novoValor < 1.0) {
						currentLogic.setPercentagem(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser maior que zero e menor do que um", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
		);
		opcoesPanel.add(percMut);
    	
    	opcoesPanel.add(new JLabel("Numero de Intervalos p/ cruzamento: "));
    	JSpinner numInterv = new JSpinner(new SpinnerNumberModel(currentLogic.getNumIntervalos(), 0, 1000, 1));
    	numInterv.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setNumIntervalos(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser positivo", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(numInterv);
    	
    	opcoesPanel.add(new JLabel("Numero de Geracoes: "));
    	JSpinner numGer = new JSpinner(new SpinnerNumberModel(currentLogic.getNumGeracoes(), 0, 1000, 1));
    	numGer.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					JSpinner spinner = (JSpinner) e.getSource();
					
					int novoValor = (Integer)spinner.getValue();
					if (novoValor > 0) {
						currentLogic.setNumGeracoes(novoValor);
					} else {
						JOptionPane.showMessageDialog(frameOpcoes,
								"Valor deve ser positivo", "Erro!!",
								JOptionPane.ERROR_MESSAGE);
					}
				}
			}
    	);
    	opcoesPanel.add(numGer);
    	
    	arvoresPanel.add(opcoesPanel, BorderLayout.CENTER);
    	
        //Sim, isso eh uma STA.
        arvoresPanel.add(Box.createHorizontalStrut(STASIZE), BorderLayout.SOUTH);
        
    	return arvoresPanel;
	}
    
    private JPanel criarRadioPanel(String desc, String nome, int numItems, ActionListener actionListener) {
    	JRadioButton[] radioButtons = new JRadioButton[numItems];
    	ButtonGroup group = new ButtonGroup();
    	
    	for (int i = 0; i < radioButtons.length; i++) {
            radioButtons[i] = new JRadioButton(nome + " - " + i);
            radioButtons[i].setActionCommand("" + i);
            radioButtons[i].addActionListener(actionListener);
            group.add(radioButtons[i]);
    	}
    	
    	radioButtons[0].setSelected(true);
    	
    	return criaPanel(desc, radioButtons);
    }
    
    private JPanel criaPanel(String desc, JComponent[] components) {
        int numChoices = components.length;
        JPanel panel = new JPanel(new BorderLayout());
        
        panel.add(new JLabel(desc), BorderLayout.NORTH);
        
        //Sim, isso é uma STA.
        panel.add(Box.createHorizontalStrut(STASIZE), BorderLayout.SOUTH);
        
        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.PAGE_AXIS));

        for (int i = 0; i < numChoices; i++) {
        	box.add(components[i]);
        }

        panel.add(box, BorderLayout.CENTER);
        return panel;
    }
    
    public void aparecer() {
		oldLogic = new Logic(currentLogic);
		frameOpcoes.setVisible(true);
    }
    
    public Logic getLogic() {
		return currentLogic;
	}
    
    private class HeuristicaFactory implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				int command = Integer.parseInt(e.getActionCommand());
				HeuristicaStrategy heuristicaStrategy;
				switch (command) {
				case 0:
					heuristicaStrategy = new Heuristica1();
					break;
				case 1:
					JOptionPane.showMessageDialog(frameOpcoes, "Nao implementada, usando 0");
					heuristicaStrategy = new Heuristica1();
					break;
				case 2:
					JOptionPane.showMessageDialog(frameOpcoes, "Nao implementada, usando 0");
					heuristicaStrategy = new Heuristica1();
					break;
				default:
					heuristicaStrategy = new Heuristica1();
					break;
				}
				
				currentLogic.setHeuristicaStrategy(heuristicaStrategy);
			} catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(frameOpcoes, "ERRO: " + nfe.toString(),
						"ERRO", JOptionPane.ERROR_MESSAGE);
				nfe.printStackTrace();
			}
		}
    }
    
    private class SeletorFactory implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				int command = Integer.parseInt(e.getActionCommand());
				SelecionadorStrategy seletorStrategy;
				switch (command) {
				case 0:
					seletorStrategy = new Selecionador1();
					break;
				case 1:
					JOptionPane.showMessageDialog(frameOpcoes, "Nao implementada, usando 0");
					seletorStrategy = new Selecionador1();
					break;
				case 2:
					JOptionPane.showMessageDialog(frameOpcoes, "Nao implementada, usando 0");
					seletorStrategy = new Selecionador1();
					break;
				default:
					seletorStrategy = new Selecionador1();
					break;
				}
				
				currentLogic.setSelecionadorStrategy(seletorStrategy);
			} catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(frameOpcoes, "ERRO: " + nfe.toString(),
						"ERRO", JOptionPane.ERROR_MESSAGE);
				nfe.printStackTrace();
			}
		}
    }
    
    private class ReprodutorFactory implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			try {
				int command = Integer.parseInt(e.getActionCommand());
				ReprodutorStrategy reprodutorStrategy;
				switch (command) {
				case 0:
					reprodutorStrategy = new Reprodutor1();
					break;
				case 1:
					JOptionPane.showMessageDialog(frameOpcoes, "Nao implementada, usando 0");
					reprodutorStrategy = new Reprodutor1();
					break;
				case 2:
					JOptionPane.showMessageDialog(frameOpcoes, "Nao implementada, usando 0");
					reprodutorStrategy = new Reprodutor1();
					break;
				default:
					reprodutorStrategy = new Reprodutor1();
					break;
				}
				
				currentLogic.setReprodutorStrategy(reprodutorStrategy);
			} catch (NumberFormatException nfe) {
				JOptionPane.showMessageDialog(frameOpcoes, "ERRO: " + nfe.toString(),
						"ERRO", JOptionPane.ERROR_MESSAGE);
				nfe.printStackTrace();
			}
		}
    }
}