package escalonadores;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import simulador.Execucao;
import simulador.Maquina;
import simulador.Relogio;
import simulador.Tarefa;

public class Genetico implements EscalonadorIF {
	
	private Collection maquinas, tarefas;
	
	private List[] cromossomos;
	private int[]  fitness;
	
	private int ultimo_melhor;
	private int contador_ultimos_melhores;
	
	public Genetico() {
		cromossomos = new List[15];
		fitness     = new int[15];
		for(int i = 0; i < 15; i++)
			fitness[i] = Integer.MAX_VALUE;
	}

	public List<Execucao> getExecucoes(Collection maquinas, Collection tarefas) {
		
		this.maquinas = maquinas;
		this.tarefas = tarefas;
		
		ultimo_melhor = Integer.MAX_VALUE;
		contador_ultimos_melhores = 0;
		
		criacao();
		
		while( contador_ultimos_melhores < 3 ) {
			
			if(ultimo_melhor == fitness[0])	{
				contador_ultimos_melhores++;
			} else {
				ultimo_melhor = fitness[0];
				contador_ultimos_melhores = 0;
			}
			
			System.out.println("\n --- Melhor Fitness: " + ultimo_melhor + "\n");
			
			cruzamento();
			mutacao();
		}
		
		return cromossomos[0];
	}

	public String getNome() {
		return "Genetico";
	}
	
	private void criacao() {
		
		System.out.println("\n\t\t --- Iniciando o processo de Criacao");
		
		for(int i = 0; i < 10; i++) {
			cromossomos[i] = cria_cromossomo_random();
			System.out.println("\t\t\t -- Cromossomo Criado: " + cromossomos[i]);
		}
		fitness();
		ordena();
	}
	
	private void cruzamento() {
		
		System.out.println("\n\t\t --- Iniciando o processo de Cruzamento");
		
		int cont = 10;
		for(int i = 0; i < 5; i++) {
			int c1_index = (int)(Math.random() * 10);
			int c2_index = (int)(Math.random() * 10);
			List<Execucao> cromossomo_resultante = gera_descendente( cromossomos[c1_index], cromossomos[c2_index] );
			cromossomos[cont++] = cromossomo_resultante;
			
			System.out.println("\t\t\t -- Cruzamento Cromossomo 1: " + cromossomos[c1_index]);
			System.out.println("\t\t\t -- Cruzamento Cromossomo 2: " + cromossomos[c2_index]);
			System.out.println("\t\t\t -- Cromossomo Descendente Gerado: " + cromossomo_resultante);
			
		}
		fitness();
		ordena();
	}
	
	private void mutacao() {
		
		System.out.println("\n\t\t --- Iniciando o processo de Mutacao");
		
		for(int i = 14; i >= 10; i--) {
			List<Execucao> cromossomo = cromossomos[i];
			
			System.out.println( "\t\t\t -- Cromossomo vai sofrer mutacao: " + cromossomo);
			
			for(int j = 0; j < cromossomo.size() / 3; j++) {
				int index1 = (int)(Math.random() * cromossomo.size());
				int index2 = (int)(Math.random() * cromossomo.size());
				
				while(index2 == index1)
					index2 = (int)(Math.random() * cromossomo.size());
				
				if(index1 > index2) {
					int temp = index2;
					index2 = index1;
					index1 = temp;
				}
				
				Execucao e1 = cromossomo.remove(index1);
				Execucao e2 = cromossomo.remove(index2 - 1);
				Execucao novo_e1 = new Execucao( e1.getId(), e1.getMaquina(), e2.getTarefa() );
				Execucao novo_e2 = new Execucao( e2.getId(), e2.getMaquina(), e1.getTarefa() );
				cromossomo.add(novo_e1);
				cromossomo.add(novo_e2);
			}
			
			System.out.println( "\t\t\t -- Cromossomo apos sofrer mutacao: " + cromossomo);
			
		}
		fitness();
		ordena();
	}
	
	private void fitness() {
		
		System.out.println("\n\t\t --- Calculando Fitness de todos cromossomos");
		
		for(int i = 0; i < 15; i++) {
			if( cromossomos[i] != null ) {
				System.out.println("\t\t\t -- Calculando Fitness do cromossomo: " + cromossomos[i]);
				fitness[i] = calcula_fitness( cromossomos[i] );
				System.out.println("\t\t\t -- Fitness do cromossomo " + i + ": " + fitness[i]);
			}
		}
		
		System.out.println("\n\t\t --- Zerando todas Execucoes de todos cromossomos");
		
		for(int i = 0; i < 15; i++) {
			if( cromossomos[i] != null ) {
				for(int j = 0; j < cromossomos[i].size(); j++ ) {
					Execucao e = (Execucao) cromossomos[i].get(j);
					e.zera();
				}
			}
		}
	}
	
	private List<Execucao> gera_descendente( List<Execucao> crom1, List<Execucao> crom2 ) {
		List<Execucao> resultante = new LinkedList<Execucao>();
		List<Execucao> clone_crom1 = new LinkedList<Execucao>(crom1);
		List<Execucao> clone_crom2 = new LinkedList<Execucao>(crom2);
		
		for(int i = 0; i < clone_crom1.size(); i++) {
			Execucao crom1_exe = clone_crom1.get(i);
			Execucao crom2_exe = clone_crom2.remove( ((int)(Math.random() * clone_crom2.size())) );
			resultante.add( new Execucao(i+1, crom1_exe.getMaquina(), crom2_exe.getTarefa()) );
		}
		
		return resultante;
	}
	
	private List<Execucao> cria_cromossomo_random() {
		List<Execucao> cromossomo = new LinkedList<Execucao>();
		List maquinas_list = new LinkedList( maquinas );
		List tarefas_list = new LinkedList( tarefas );
		
		int cont_execucao_id = 1;
		int cont_maquina_id = 0;
		while(tarefas_list.size() > 0) {
			int tarefa_index = (int)(Math.random() * tarefas_list.size());
			Maquina m = (Maquina) maquinas_list.get(cont_maquina_id++);
			Tarefa t = (Tarefa) tarefas_list.remove( tarefa_index );
			Execucao e = new Execucao(cont_execucao_id++, m, t);
			cromossomo.add( e );
			if(cont_maquina_id >= maquinas_list.size())
				cont_maquina_id = 0;
		}
		
		return cromossomo;
	}
	
	private void ordena() {
		
		System.out.println("\n\t\t --- Ordenando");
		
		for(int i = 0; i < cromossomos.length-1; i++) {
			for(int j = i+1; j < cromossomos.length; j++ ) {
				if( fitness[i] > fitness[j] ) {
					int temp = fitness[i];
					fitness[i] = fitness[j];
					fitness[j] = temp;
					List<Execucao> list_temp = cromossomos[i];
					cromossomos[i] = cromossomos[j];
					cromossomos[j] = list_temp;
				}
			}
		}
	}
	
	private int calcula_fitness( List<Execucao> cromossomo ) {
		
		Relogio relogio = new Relogio();
		List<Execucao> execucoes_na_fila = new LinkedList<Execucao>(cromossomo);
		List<Execucao> execucoes_acabadas = new LinkedList<Execucao>();
		List<Execucao> execucoes_escalonadas = new LinkedList<Execucao>();
		
		Iterator maquinas_it = maquinas.iterator();
		while(maquinas_it.hasNext()) {
			Maquina m = (Maquina) maquinas_it.next();
			Execucao e = get_prox_execucao_da_maquina(m, execucoes_na_fila);
			if( e != null ) {
				execucoes_na_fila.remove( e );
				e.getMaquina().ocupa();
				execucoes_escalonadas.add( e );
				//System.out.println("Tempo " + relogio.getUts() + " - Execucao Escalonada: " + e.toString());
			}
		}
		
		while( execucoes_acabadas.size() < cromossomo.size() ) {
			
			relogio.incrementa_uts();
			
			for(int i = 0; i < execucoes_escalonadas.size(); i++) {
				Execucao execucao = execucoes_escalonadas.get(i);
				if(execucao.terminada()) continue;
				execucao.add_uts();		
				if(execucao.getUts_restantes() <= 0) {
					execucao.termina();
					execucoes_acabadas.add(execucao);
					
					//System.out.println("Tempo " + relogio.getUts() + " - Execucao Terminada: " + execucao.toString());
					
					Execucao execucao_proxima = get_prox_execucao_da_maquina( execucao.getMaquina(), execucoes_na_fila );
					if(execucao_proxima == null) {
						execucao.getMaquina().desocupa();
					} else {
						execucao_proxima.uts();
						execucoes_escalonadas.add( execucao_proxima );
					}
					
					//if(execucao_proxima != null)
						//System.out.println("Tempo " + relogio.getUts() + " - Execucao Escalonada: " + execucao_proxima.toString());
					
				}
			}
			
		}
		
		//System.out.println();
		
		return relogio.getUts();
	}
	
	private Execucao get_prox_execucao_da_maquina(Maquina maquina, List<Execucao> execucoes) {
		Iterator<Execucao> execucoes_it = execucoes.iterator();
		while(execucoes_it.hasNext()) {
			Execucao execucao = execucoes_it.next();
			if( maquina.equals( execucao.getMaquina() ) ) {
				execucoes_it.remove();
				return execucao;
			}
		}
		return null;
	}

}















