package nom.paolo.ai.pathfinder;
import java.util.LinkedList;
import java.util.List;

public class CartesianPathFinder implements PathFinder {
	private int tileSize;
	private PathFinder pathFinder;
	
	public CartesianPathFinder( PathFinder pathFinder, int tileSize ) {
		this.pathFinder = pathFinder;
		this.tileSize = tileSize;
	}
	
	public List<PathComponent> findPath(int startX, int startY, int targetX, int targetY ) {
		List<PathComponent> path = pathFinder.findPath(startX / tileSize, startY / tileSize, targetX / tileSize, targetY / tileSize );
		
		if( path == null || path.size() == 1 ) {
			return path;
		}
				
		List<PathComponent> reducedPath = new LinkedList<PathComponent>();
		reducedPath.add( path.get(0) );
		
		for( int i = 0; i < path.size() - 1; ) {			
			PathComponent actualStep = path.get(i);
			PathComponent nextStep = path.get(i + 1);
			
			int next = i;
			for( int k = i + 1; k < path.size(); k++ ) {
				if( lineMayBeDraw( actualStep, path.get(k), pathFinder.getMap() )  ) {
					nextStep = path.get(k);
					next = k;
				}
			}
			i = next;
			
			reducedPath.add( nextStep );
		}
		
		for( int k = 0; k < reducedPath.size(); k++ ) {
			reducedPath.add( reducedPath.remove(0) );
		}
		
		return reducedPath;
	}
	
	private boolean lineMayBeDraw(PathComponent p1, PathComponent p2, int[][] map ) {
		int dx = Math.abs( p1.getX() - p2.getX() ) * tileSize;
		int dy = Math.abs( p1.getY() - p2.getY() ) * tileSize;
		
		int x1 = p1.getX() * tileSize + (tileSize / 2);
		int y1 = p1.getY() * tileSize + (tileSize / 2);
			
		int xInc = 0;
		if( p2.getX() < p1.getX() ) { 
			xInc = -1;
		} else if( p2.getX() > p1.getX() ) {
			xInc = 1;	
		}
		
		int yInc = 0;
		if( p2.getY() < p1.getY() ) { 
			yInc = -1;
		} else if( p2.getY() > p1.getY() ) { 
			yInc = 1;
		}
		
		for( int x = x1, w = 0; w <= dx ; x += xInc, w += 1 ) {
			for( int y = y1, h = 0; h <= dy ; y += yInc, h += 1 ) {
				if( map[ y / tileSize ][ x / tileSize ] != 0 ) {					
					return false;
				}
			}
		}
		
		return true;
	}

	public int[][] getMap() {
		return this.pathFinder.getMap();
	}
}
