package nom.paolo.ai.util;

/**
 * Implements simple math operations, such as the distance between two points 
 * 
 * @author Paolo Victor
 */
public class MathUtils {
	/**
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @return The distance between (x1,y1) and (x2,y2)
	 */
	public static double distanceBetween( double x1, double y1, double x2, double y2 ) {
		double dx = x1 - x2;
		double dy = y1 - y2;
		
		return Math.sqrt( dx * dx + dy * dy );
	}
	
	/**
	 * Gets the angle between two angles, using the vector's dot product.
	 * 
	 * @param theta1
	 * @param theta2
	 * 
	 * @return The angle between two angles
	 */
	public static double angleBetween( double theta1, double theta2 ) {
		// X . Y == |X| * |Y| * cos(theta),
		// considering:
		// |X| = 1
		// |Y| = 1
		// so X = ( cos( theta1 ) , sin( theta1 ) )
		//    Y = ( cos( theta2 ) , sin( theta2 ) )
		//    X . Y = Xx * Yx + Xy * Yy
		//    theta = arccos( Xx * Yx + Xy * Yy )
		double Xx = Math.cos( theta1 );
		double Xy = Math.sin( theta1 );
		
		double Yx = Math.cos( theta2 );
		double Yy = Math.sin( theta2 );
		
		return Math.acos( Xx * Yx + Xy * Yy );
	}
	
	public static void main( String args[] ) {
		System.out.println( Math.toDegrees( MathUtils.angleBetween( Math.toRadians( 360 ), Math.toRadians( 200 ) ) ) );
	}
}
