package simulador;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import escalonadores.EscalonadorIF;

public class Simulacao {
	
	private Collection<Tarefa> tarefas;
	private Collection<Maquina> maquinas;
	private EscalonadorIF escalonador;
	
	public Simulacao(EscalonadorIF escalonador, Collection<Tarefa> tarefas, Collection<Maquina> maquinas) {
		this.escalonador = escalonador;
		this.tarefas = tarefas;
		this.maquinas = maquinas;
	}
	
	public int simula() {
		
		System.out.println("Inicio da simulacao");
		System.out.println("Escalonador: " + escalonador.getNome());
		System.out.println("Numero de Maquinas: " + maquinas.size());
		System.out.println("Numero de Tarefas: " + tarefas.size());
		System.out.println();
		
		Relogio relogio = new Relogio();
		List<Execucao> execucoes = escalonador.getExecucoes(maquinas, tarefas);
		List<Execucao> execucoes_na_fila = new LinkedList<Execucao>(execucoes);
		List<Execucao> execucoes_acabadas = new LinkedList<Execucao>();
		List<Execucao> execucoes_escalonadas = new LinkedList<Execucao>();
		
		Iterator maquinas_it = maquinas.iterator();
		while(maquinas_it.hasNext()) {
			Maquina m = (Maquina) maquinas_it.next();
			Execucao e = get_prox_execucao_da_maquina(m, execucoes_na_fila);
			if( e != null ) {
				execucoes_na_fila.remove( e );
				e.getMaquina().ocupa();
				execucoes_escalonadas.add( e );
				System.out.println("Tempo " + relogio.getUts() + " - Execucao Escalonada: " + e.toString());
			}
		}
		
//		for(int i = 0; i < maquinas.size() && i < tarefas.size(); i++) {
//			Execucao execucao = execucoes_na_fila.remove(0);
//			execucao.getMaquina().ocupa();
//			execucoes_escalonadas.add( execucao );
//			
//			System.out.println("Tempo " + relogio.getUts() + " - Execucao Escalonada: " + execucao.toString());
//		}
		
		while( execucoes_acabadas.size() < execucoes.size() ) {
			
			relogio.incrementa_uts();
			
			for(int i = 0; i < execucoes_escalonadas.size(); i++) {
				Execucao execucao = execucoes_escalonadas.get(i);
				execucao.add_uts();
				if(execucao.getUts_restantes() == 0) {
					execucao.termina();
					execucoes_acabadas.add(execucao);
					
					System.out.println("Tempo " + relogio.getUts() + " - Execucao Terminada: " + execucao.toString());
					
					Execucao execucao_proxima = get_prox_execucao_da_maquina( execucao.getMaquina(), execucoes_na_fila );
					if(execucao_proxima == null) {
						execucao.getMaquina().desocupa();
					} else {
						execucao_proxima.uts();
						execucoes_escalonadas.add( execucao_proxima );
					}
					
					if(execucao_proxima != null)
						System.out.println("Tempo " + relogio.getUts() + " - Execucao Escalonada: " + execucao_proxima.toString());
					
				}
			}
			
		}
		
		System.out.println();
		
		return relogio.getUts();
	}
	
	private Execucao get_prox_execucao_da_maquina(Maquina maquina, List<Execucao> execucoes) {
		Iterator<Execucao> execucoes_it = execucoes.iterator();
		while(execucoes_it.hasNext()) {
			Execucao execucao = execucoes_it.next();
			if( maquina.equals( execucao.getMaquina() ) ) {
				execucoes_it.remove();
				return execucao;
			}
		}
		return null;
	}

}
