package nom.paolo.ai;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JPanel;

import nom.paolo.ai.game.GameCanvas;
import nom.paolo.ai.game.objects.Agent;
import nom.paolo.ai.util.ExperimentStats;

public class Main extends JFrame implements MouseListener {
	JPanel panel;
	GameCanvas canvas = null;
	
	public boolean isDoubleBuffered() {
		return true;
	}
	
	public Main() throws IOException, InstantiationException {
		super("AI");
		
		this.setResizable(false);
		this.setSize( 512, 400 );	
		this.setVisible( true );
		
		this.panel = new JPanel();
		this.panel.setSize( 512, 400 );		
		this.getContentPane().add( panel );
				
		this.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e){
					System.exit(0);
				}
		    }
		);	
		
		this.addMouseListener( this );
	}
	
	private void run( int numExperiments ) throws IOException {
		int n = 0;
		int blueWins = 0;
		int redWins = 0;
		int greenWins = 0;
		int yellowWins = 0;
		
		BufferedImage buffer = (BufferedImage)panel.createImage( this.getWidth(), this.getHeight() );
		
		while( n < numExperiments ) {
			System.out.println("Run: " + (n+1));
			n++;			
		
			canvas = new GameCanvas( "defs.txt", 512, 400, buffer );		
			canvas.startAgents();		
		
			while( ! canvas.isGameOver() ) {			
				canvas.display();
				this.panel.getGraphics().drawImage( buffer, 0, 0, this );
				
				try {
					Thread.sleep( 100/6 );
				} catch (InterruptedException e1) {
				}
			}
			
			if( canvas.hasTeamWon( Agent.TEAM_BLUE ) ) {
				System.out.println("Blue wins!");
				blueWins++;
			} else if( canvas.hasTeamWon( Agent.TEAM_RED ) ) {
				System.out.println("Red wins!");
				redWins++;
			} else if( canvas.hasTeamWon( Agent.TEAM_GREEN ) ) {
				System.out.println("Green wins!");
				greenWins++;
			} else if( canvas.hasTeamWon( Agent.TEAM_YELLOW ) ) {
				System.out.println("Yellow wins!");
				yellowWins++;
			}
		}
		
		System.out.println( "B: " + blueWins );
		System.out.println( "R: " + redWins );
		System.out.println( "G: " + greenWins );
		System.out.println( "Y: " + yellowWins );
		
		System.out.println( "Target locks: " + ExperimentStats.getInstance().getTargetLocks() );
		System.out.println( "Friendly target locks: " + ExperimentStats.getInstance().getFriendTargetLocks() );
	}

	public static void main( String args[] ) throws IOException, InstantiationException {
		Main instance = new Main();
		instance.run( 10 );
	}

	public void mouseClicked(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseReleased(MouseEvent e) {
		if( this.canvas != null ) {
			this.canvas.cycleCamera();
		}
	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
}
