package nom.paolo.ai.game.animation;

import java.awt.Image;
import java.awt.image.BufferedImage;

/**
 * Implements an animation
 * 
 * @author Paolo Victor
 */
public class Animation {	
	protected int width;
	protected int height;
	
	protected int actualFrame;
	protected int restartFrame;
	protected int initialFrame;
	protected int numberOfFrames;	
	
	protected int delay;
	protected int ticks;
	
	protected BufferedImage[] subImages;
	
	public Animation( BufferedImage image, int width, int height, int initialFrame, int restartFrame, int numberOfFrames, int delay ) {		
		this.width = width;
		this.height = height;
		
		this.initialFrame = initialFrame;
		this.actualFrame = initialFrame;
		this.restartFrame = restartFrame;
		this.numberOfFrames = numberOfFrames;
		
		this.delay = delay;
		this.ticks = delay;
		
		this.loadSubImages( image, numberOfFrames, width, height );
	}
	
	private void loadSubImages( BufferedImage image, int n, int w, int h ) {
		this.subImages = new BufferedImage[n];
		
		for( int x = 0; x < n; x++ ) {
			this.subImages[x] = image.getSubimage( x * width, 0, width, height );
		}
	}

	public void reset() {
		this.actualFrame = this.initialFrame;
	}
	
	public Image getActualFrame() {
		return this.subImages[ actualFrame ];
	}

	public void update() {
		this.ticks--;
		
		if( this.ticks <= 0 ) {
			this.actualFrame++;
			if( this.actualFrame >= this.numberOfFrames ) {
				this.actualFrame = this.restartFrame;
			}
			
			this.ticks = this.delay;
		}
	}
}
