package nom.paolo.ai.game.animation;

import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.util.HashMap;
import java.util.Map;

import nom.paolo.ai.game.display.Viewport;

/**
 * Used as an avatar for a game object, such as an agent
 * 
 * @author Paolo Victor
 */
public class BaseSprite implements Sprite {
	protected double x;
	protected double y;
	protected double angle;
	
	protected int width;
	protected int height;
	
	protected Animation currentAnimation;
	protected Map<String, Animation> animations;
	AffineTransform atx = new AffineTransform();
	
	public BaseSprite( int x, int y, int width, int height, double angle ) {
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
		this.angle = angle;
		
		this.animations = new HashMap<String, Animation>();
	}
	
	public double getPositionX() {
		return this.x;
	}

	public double getPositionY() {
		return this.y;
	}
	
	public double getAngle() {
		return this.angle;
	}

	public Image getImage() {
		if( this.currentAnimation == null ) {
			return null;
		}
		
		return this.currentAnimation.getActualFrame();
	}

	public int getDisplayPositionX() {
		return (int)this.getPositionX();
	}

	public int getDisplayPositionY() {
		return (int)this.getPositionY();
	}

	public void updateObject() {
		this.updateAnimation();
	}

	public void setAnimation(String animationName) {
		this.currentAnimation = animations.get( animationName );
		this.currentAnimation.reset();
	}
	
	public void attachAnimation( String animationName, Animation animation ) {
		this.animations.put( animationName, animation );
	}

	public void updateAnimation() {
		this.currentAnimation.update();
	}

	public double getRotateAngle() {
		return this.getAngle();
	}

	public int getWidth() {
		return this.width;
	}

	public int getHeight() {
		return this.height;
	}

	public int compareTo(Object o) {
		return 0;
	}

	public void displayInfo(Viewport viewPort) {		
	}
}
