package nom.paolo.ai.game.display;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import nom.paolo.ai.game.map.GameMap;


public class Viewport2D implements Viewport {
	protected Graphics2D graphics = null;
	protected AffineTransform transform =  new AffineTransform();
	protected Displayable lockedObject = null;
	private int width;
	private int height;
	
	public Viewport2D( Graphics graphics, int width, int height ) {
		this.graphics = (Graphics2D)graphics;
		this.graphics.setClip( 0, 0, width, height );
		this.graphics.setFont( new Font("Arial", Font.BOLD, 14) );		
		
		this.width = width;
		this.height = height;
	}
	
	public void setGraphics( Graphics graphics ) {
		this.graphics = (Graphics2D)graphics;
		this.graphics.setClip( 0, 0, width, height );
	}
	
	public void drawObjects( List<Displayable> objects ) {
		graphics.setColor( Color.BLACK );
		graphics.fillRect( 0, 0, width, height );
		
		GameMap.getInstance().displayAt( graphics, -this.getXOffset(), -this.getYOffset() );
		
		Set<Displayable> displayableObjects = new TreeSet<Displayable>( objects );
		
		if( this.graphics != null ) {
			for( Displayable object : displayableObjects ) {
				Image image = object.getImage();
				
				if( image != null ) {					
					transform.setToTranslation( object.getDisplayPositionX() - object.getWidth() / 2 - this.getXOffset(), object.getDisplayPositionY() - object.getHeight() / 2 - this.getYOffset() );
					transform.rotate( object.getRotateAngle(), object.getWidth(), object.getHeight() );
					graphics.drawImage(  image, transform, null );
				}
				
				object.displayInfo( this );
			}
		}
	}

	private int getXOffset() {
		if( this.lockedObject != null ) {
			return this.lockedObject.getDisplayPositionX() - this.width/2;
		}		
		return 0;
	}

	private int getYOffset() {
		if( this.lockedObject != null ) {
			return this.lockedObject.getDisplayPositionY() - this.height/2;
		}		
		
		return 0;
	}

	public void lockOn(Displayable object) {
		this.lockedObject = object;
	}
	
	public void drawString(String string, double x, double y, Color color, boolean enableOffset ) {
		this.graphics.setColor( color );
		if( enableOffset ) {
			this.graphics.drawString( string, (int)x - this.getXOffset(), (int)y - this.getYOffset() );
		} else {
			this.graphics.drawString( string, (int)x, (int)y );			
		}
	}
	
	public void drawString(String string, double x, double y, Color color) {
		this.drawString( string, x, y, color, true );
	}

	public void drawLine(int x, int y, int x2, int y2, Color color) {
		this.graphics.setColor( color );
		this.graphics.drawLine( (int)x - this.getXOffset(), (int)y - this.getYOffset(), (int)x2 - this.getXOffset(), (int)y2 - this.getYOffset() );
	}
	
	public void fillRect( int x, int y, int w, int h, Color color ) {
		this.fillRect( x, y, w, h, color, true );
	}
	
	public void fillRect( int x, int y, int w, int h,  Color color, boolean enableOffset ) {
		this.graphics.setColor( color );
		
		if( enableOffset ) {
			this.graphics.fillRect( x - this.getXOffset(), y - this.getYOffset(), w, h );
		} else {
			this.graphics.fillRect( x, y, w, h );			
		}
	}
	
	public void rect( int x, int y, int w, int h, Color color ) {
		this.graphics.setColor( color );
		this.graphics.drawRect( x - this.getXOffset(), y - this.getYOffset(), w, h );		
	}

	public int getWidth() {
		return this.width;
	}
}
