package nom.paolo.ai.game.map;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import javax.imageio.ImageIO;

import nom.paolo.ai.game.objects.Agent;
import nom.paolo.ai.util.MathUtils;

public class GameMap {

	private static final int SPAWN_POINT_BLUE = 6;

	private static final int SPAWN_POINT_RED = 7;
	
	private static final int SPAWN_POINT_GREEN = 8;

	private static final int SPAWN_POINT_YELLOW = 9;

	protected static GameMap instance = null;
	
	protected int tileSize;

	protected int[][] map;
	protected int[][][] gameMap;
	
	protected Image[] tiles;
	
	protected BufferedImage background;
	protected BufferedImage buffer;
	
	private Map< Integer, List<Point2D> > spawnPoints;
	
	private GameMap() {		
	}
	
	public static GameMap getInstance() {
		if( instance == null ) {
			instance = new GameMap();
		}
		
		return instance;
	}
	
	public void loadMap( String string ) throws FileNotFoundException {
		Scanner scanner = new Scanner( new File( string ) );
			
		String tileset = scanner.nextLine();
		
		this.spawnPoints = new HashMap< Integer, List<Point2D> >();
		
		this.tileSize = scanner.nextInt(); 
		int width = scanner.nextInt();
		int height = scanner.nextInt();
		
		this.map = new int[height][width];
		
		for( int y = 0; y < height; y++ ) {
			for( int x = 0; x < width; x++ ) {			
				
				int nextValue = scanner.nextInt();
				
				if( nextValue == SPAWN_POINT_BLUE ) {
					this.addSpawpoint( Agent.TEAM_BLUE, new Point2D.Double( x * tileSize + tileSize/2, y * tileSize + tileSize/2 ) );
					this.map[y][x] = 0;
				} else if( nextValue == SPAWN_POINT_RED ) {
					this.addSpawpoint( Agent.TEAM_RED, new Point2D.Double( x * tileSize + tileSize/2, y * tileSize + tileSize/2 ) );
					this.map[y][x] = 0;
				} else if( nextValue == SPAWN_POINT_GREEN ) {
					this.addSpawpoint( Agent.TEAM_GREEN, new Point2D.Double( x * tileSize + tileSize/2, y * tileSize + tileSize/2 ) );
					this.map[y][x] = 0;
				} else if( nextValue == SPAWN_POINT_YELLOW ) {
					this.addSpawpoint( Agent.TEAM_YELLOW, new Point2D.Double( x * tileSize + tileSize/2, y * tileSize + tileSize/2 ) );
					this.map[y][x] = 0;
				} else {
					this.map[y][x] = nextValue;
				}				
			}
		}
		
		try {			
			background = ImageIO.read( new File( tileset ) );
			buffer = new BufferedImage( width * tileSize, height * tileSize, BufferedImage.TYPE_INT_ARGB );
			
			tiles = new Image[4];
			tiles[0] = background.getSubimage( 0, 0, tileSize, tileSize );
			tiles[1] = background.getSubimage( 0, tileSize, tileSize, tileSize );
			tiles[2] = background.getSubimage( 0, 2 * tileSize, tileSize, tileSize );
			tiles[3] = background.getSubimage( 0, 3 * tileSize, tileSize, tileSize );
			
			this.drawMap( buffer.getGraphics() );
		} catch (IOException e) {
			background = null;
			e.printStackTrace();
		}
	}

	private void addSpawpoint( int teamId, Point2D point ) {
		if( ! this.spawnPoints.containsKey( teamId ) ) {
			this.spawnPoints.put( teamId, new LinkedList<Point2D>() );
		}
		
		this.spawnPoints.get( teamId ).add( point );
	}

	public int getTileSize() {
		return tileSize;
	}

	public int[][] getVisibleMap(int teamId) {
		return map;
	}

	public void displayAt(Graphics graphics, int x, int y) {
		graphics.drawImage( buffer, x, y, null );
	}

	private void drawMap( Graphics graphics ) {
		int[][] m = this.getVisibleMap( 0 );
		
		for( int y = 0; y < m.length; y++ ) {
			for( int x = 0; x < m[y].length; x++ ) {
				if( background == null ) {
					if( m[y][x] > 0 ) {
						graphics.setColor( Color.RED );
					} else {
						graphics.setColor( Color.WHITE );
					}
					
					graphics.fillRect( x * tileSize, y * tileSize, tileSize, tileSize );
				} else {
					graphics.drawImage( tiles[ m[y][x] ], x * tileSize, y * tileSize, null );
				}
			}
		}
	}

	public boolean lineMayBeDraw( double x1, double y1, double x2, double y2, int teamId ) {		
		double distance = MathUtils.distanceBetween( x1, y1, x2, y2 );
		double xIncUnit = Math.abs( x1 - x2 ) / distance;
		double yIncUnit = Math.abs( y1 - y2 ) / distance;
				
		double xInc = 0;
		if( x2 < x1 ) { 
			xInc = -xIncUnit;
		} else if( x2 > x1 ) {
			xInc = xIncUnit;	
		}
		
		double yInc = 0;
		if( y2 < y1 ) { 
			yInc = -yIncUnit;
		} else if( y2 > y1 ) { 
			yInc = yIncUnit;
		}
		
		for( double x = x1, w = 0; w < distance ; x += xInc, w += 1 ) {
			for( double y = y1, h = 0; h < distance ; y += yInc, h += 1 ) {
				if( map[ (int)y / tileSize ][ (int)x / tileSize ] != 0 ) {
					return false;
				}
			}
		}
		
		return true;
	}

	public Point2D getSpawnPoint( int teamId ) {
		List spawnPointsList = this.spawnPoints.get( teamId );
		int spawn = (int)(Math.random() * ( spawnPointsList.size() - 1 ));
		
		return this.spawnPoints.get( teamId ).get( spawn );
	}
}
